function [t,p,s,percvar,var]=varimax(t,p,x,f)
%  
%  [T,P,S] = VARIMAX(T,P,X,F)   Function to perform varimax rotation of 
%                               eigenvectors and return the rotated 
%  eigenvectors and the scores projected on the new axis.  This function does 
%  several variations on the varimax theme including the "normalized varimax" 
%  where the PC's are first scaled by the variable's communalities, eigenvalue
%  weighted varimax, and ARTHUR varimax.
%  
%  t = on input matrix of score vectors for the normal PC's, on output
%       contain the score vectors for the rotated PC's
%  p = on input the matrix of loading vectors (eigenvectors) to be 
%       rotated, on output contains the rotated PC's
%  s = matrix containing the sum or squares for the rotated PC's and the 
%       residual
%  x = data matrix used to derive the input t and p vectors.  Must be
%       scaled in the same way as when the PC's were calculated.
%  f = flag to indicate type of varimax rotation.  Options are :
%       1 - raw varimax
%       2 - nomalized varimax
%       3 - eigenvalue weighted varimax
%       4 - Arthur varimax (eigenvalue weighted and normalized varimax)
%
%  From: Dave Veltkamp (veltkamp@cpac.washington.edu)

 [i,k]=size(t);
 [i,j]=size(x);
 sst=sum(sum(x.^2));
 w = 1;
 if f == 4 | f == 3
%  
%  Normalize eigenvectors by their eigenvalues
%  
   for kk=1:k
     ev(kk) = sqrt(sum(t(:,kk).^2) ./ (i-1));
     p(:,kk) = p(:,kk).*ev(kk);
   end
 end
 if f == 2 | f == 4
%  
% Calculate original communalities
%  
   comm = sqrt(sum(p'.^2)');
%  
%  Normalize eigenvectors by their communalities
%  
   for kk = 1:k
     p(:,kk) = p(:,kk)./comm;
   end
 end
 mv = 1;
 vvv = 0;
 nc = 0;
 ncount = 0;
%
% Start the iteration
%  
 while mv<=50
   vv = vvv;
   tmp = 0;
%  
%  z = varimax criteria to be maximized
%  
   z  = sum(((j*(sum(p.^4))) - (w * (sum(p.^2)).^2)) ./ j^2);
   vvv = z;
   tv = vvv - vv;
%  
%  Check that the difference is large enough to continue
%  
   if tv <= 0.001*vv
     nc = nc+1;
     if nc >=3
       %disp(['Completed after ',int2str(mv-3),' itterations'])
       break
      end
    end
%  
%  for each eigenvector pair compute angle of rotation 
%  phi = num/den
%  where
%  
%    num = 2 * {nvar * sum[(x^2 - y^2) * (2 * x * y)] - 
%          [sum(x^2 - y^2) * sum(2 * x * y)]}
%    den = nvar * {sum[(x^2 - y^2)^2 - (2 * x * y)^2]} - 
%          {sum[x^2 - y^2]^2 - sum[2 * x * y]^2}
%  
    for a = 1:k-1
      aa = a + 1;
      for b = aa:k
        t1 = p(:,a).^2 - p(:,b).^2;
        t2 = 2 * p(:,a).*p(:,b);
        t3 = sum(t1.^2 - t2.^2);
        t4 = sum(t1.*t2);
        t5 = sum(t1);
        t6 = sum(t2);
        num = 2 * ((j * t4) - (t5 * t6));
        den = (j * t3) - ((t5^2) - (t6^2));
%  
%  calculate the angle in radians
%  
        phi = atan2(num,den) / 4;
%  
%  if angle is great enough go ahead and do the rotation
%  
        if abs(num)+abs(den) > 0.0001 & abs(phi) >= 0.000025
          cosp = cos(phi);
          sinp = sin(phi);
          ncount = 0;
          tmp = (p(:,a).*cosp) + (p(:,b).*sinp);
          p(:,b) = (-p(:,a).*sinp) + (p(:,b).*cosp);
          p(:,a) = tmp;
         else
%  
%  otherwise just increment the counter and go back for the next pair
%  
           ncount = ncount + 1;
%  
%  if we have done all pairs and no rotation is done exit the iteration
%  
           if ncount > ((k-1) * k) / 2
             break;break;break
           end
         end
       end
   end
   mv = mv + 1;
 end
 if mv > 50
   error(['Did not converge after ',int2str(mv),' itterations'])
 end
 if f == 2 | f == 4
%  
%  Denormalize rotated eigenvectors by re-multiplying them by the communalities
%
   for kk=1:k
     p(:,kk) = p(:,kk) .* comm;
   end
 end
%  
%  calculate the new communalities ( not used )
%  
%  newcom = sqrt(diag(p * p'))
%  
%  renormalize eigenvectors
%  
 for kk=1:k
   var(kk) = (sum(p(:,kk).^2));
   if f ==3 | f == 4
     p(:,kk) = p(:,kk) ./ sqrt(var(kk));
   end
 end
%  
% project new scores
%  
 if f == 1 | f == 2
   t = x*p;
 elseif f == 3 | f == 4
   t = x * pinv(p)';
 end
%  
%  estimate amount of variance described by each rotated vector
%  
 for kk=1:k
   new = norm(t(:,kk),2);
   est = t(:,kk) * p(:,kk)';
   x = x - est;
   v = sum(sum(est.^2));
   s(kk,2) = (v/sst) * 100;
   s(kk,1) = (new^2)/(i-1);
 end
%  
%  sort from highest to lowest variance
%  
 [dum,indx] = sort(s(:,2));
 indx = indx(k:-1:1);
 t = t(:,indx);
 p = p(:,indx);
 var = var(indx); % The new varimax rotated eigenvalues (meaningful for f=3|f=4)
 s = s(indx,:);
 percvar=s(:,2)'; % The overall percent variance described by varimax rotated eigenvectors
